#ifndef __DESFIRE_H__
#define __DESFIRE_H__

#define APPLICATION_ID_SIZE          3
#define MAX_INFO_FRAME_SIZE          60

/*
 * DESFire commands (codes sent by the PCD)
 * ----------------------------------------
 */

#define DF_AUTHENTICATE                     0x0A
#define DF_CREDIT                           0x0C
#define DF_LIMITED_CREDIT                   0x1C
#define DF_WRITE_RECORD                     0x3B
#define DF_WRITE_DATA                       0x3D
#define DF_GET_KEY_SETTINGS                 0x45
#define DF_CHANGE_KEY_SETTINGS              0x54
#define DF_SELECT_APPLICATION               0x5A
#define DF_CHANGE_FILE_SETTINGS             0x5F
#define DF_GET_VERSION                      0x60
#define DF_GET_KEY_VERSION                  0x64
#define DF_GET_APPLICATION_IDS              0x6A
#define DF_GET_VALUE                        0x6C
#define DF_GET_FILE_IDS                     0x6F
#define DF_ABORT_TRANSACTION                0xA7
#define DF_READ_RECORDS                     0xBB
#define DF_READ_DATA                        0xBD
#define DF_CREATE_CYCLIC_RECORD_FILE        0xC0
#define DF_CREATE_LINEAR_RECORD_FILE        0xC1
#define DF_CHANGE_KEY                       0xC4
#define DF_COMMIT_TRANSACTION               0xC7
#define DF_CREATE_APPLICATION               0xCA
#define DF_CREATE_BACKUP_DATA_FILE          0xCB
#define DF_CREATE_VALUE_FILE                0xCC
#define DF_CREATE_STD_DATA_FILE             0xCD
#define DF_DELETE_APPLICATION               0xDA
#define DF_DEBIT                            0xDC
#define DF_DELETE_FILE                      0xDF
#define DF_CLEAR_RECORD_FILE                0xEB
#define DF_GET_FILE_SETTINGS                0xF5
#define DF_FORMAT_PICC                      0xFC


/*
 * DESFire status (codes returned by the PICC)
 * -------------------------------------------
 */

#define DF_OPERATION_OK                     0x00
#define DF_NO_CHANGES                       0x0C
#define DF_OUT_OF_EEPROM_ERROR              0x0E
#define DF_ILLEGAL_COMMAND_CODE             0x1C
#define DF_INTEGRITY_ERROR                  0x1E
#define DF_NO_SUCH_KEY                      0x40
#define DF_LENGTH_ERROR                     0x7E
#define DF_PERMISSION_DENIED                0x9D
#define DF_PARAMETER_ERROR                  0x9E
#define DF_APPLICATION_NOT_FOUND            0xA0
#define DF_APPL_INTEGRITY_ERROR             0xA1
#define DF_AUTHENTICATION_CORRECT           0xAC
#define DF_AUTHENTICATION_ERROR             0xAE
#define DF_ADDITIONAL_FRAME                 0xAF
#define DF_BOUNDARY_ERROR                   0xBE
#define DF_CARD_INTEGRITY_ERROR             0xC1
#define DF_COMMAND_ABORTED                  0xCA
#define DF_CARD_DISABLED_ERROR              0xCD
#define DF_COUNT_ERROR                      0xCE
#define DF_DUPLICATE_ERROR                  0xDE
#define DF_FILE_NOT_FOUND                   0xF0
#define DF_FILE_INTEGRITY_ERROR             0xF1
#define DF_EEPROM_ERROR                     0xEE

/*
 * DESFire API status/error codes
 * ------------------------------
 */
#define DFCARD_ERROR                        -1000
#define DFCARD_LIB_CALL_ERROR               DFCARD_ERROR+1
#define DFCARD_OUT_OF_MEMORY                DFCARD_ERROR+2
#define DFCARD_OVERFLOW                     DFCARD_ERROR+3
#define DFCARD_WRONG_KEY                    DFCARD_ERROR+4
#define DFCARD_WRONG_MAC                    DFCARD_ERROR+5
#define DFCARD_WRONG_CRC                    DFCARD_ERROR+6
#define DFCARD_WRONG_LENGTH                 DFCARD_ERROR+7
#define DFCARD_WRONG_PADDING                DFCARD_ERROR+8
#define DFCARD_WRONG_FILE_TYPE              DFCARD_ERROR+9
#define DFCARD_WRONG_RECORD_SIZE            DFCARD_ERROR+10
#define DFCARD_PCSC_BAD_RESP_LEN            DFCARD_ERROR+11
#define DFCARD_PCSC_BAD_RESP_SW             DFCARD_ERROR+12

/*
 * DESFire file types
 * ------------------
 */
#define	DF_STANDARD_DATA_FILE			          0x00
#define	DF_BACKUP_DATA_FILE				          0x01
#define	DF_VALUE_FILE                       0x02
#define	DF_LINEAR_RECORD_FILE			          0x03
#define	DF_CYCLIC_RECORD_FILE			          0x04

/*
 * DESFire communication mode flag combinations.
 * ---------------------------------------------
 */
#define	DF_COMM_MODE_PLAIN			            0x00
#define	DF_COMM_MODE_MACED			            0x01
#define	DF_COMM_MODE_PLAIN2			            0x02
#define	DF_COMM_MODE_ENCIPHERED		          0x03

/*
 * DESFire access rights
 * ---------------------
 */

/* Shift counts for extracting the individual AccessRight settings. */
#define DF_GET_READ_ONLY                    12
#define DF_GET_WRITE_ONLY                   8
#define DF_GET_READ_WRITE                   4

/* AccessRights masks. */
#define DF_READ_ONLY                        0xF000
#define DF_WRITE_ONLY                       0x0F00
#define DF_READ_WRITE                       0x00F0
#define DF_CHANGE_ACCESS_RIGHTS             0x000F

/*
 * DF_VERSION_INFO
 * ---------------
 * Structure for returning the information supplied by the GetVersion command.
 */
typedef struct
{
  BYTE    bHwVendorID;
  BYTE    bHwType;
  BYTE    bHwSubType;
  BYTE    bHwMajorVersion;
  BYTE    bHwMinorVersion;
  BYTE    bHwStorageSize;
  BYTE    bHwProtocol;
  BYTE    bSwVendorID;
  BYTE    bSwType;
  BYTE    bSwSubType;
  BYTE    bSwMajorVersion;
  BYTE    bSwMinorVersion;
  BYTE    bSwStorageSize;
  BYTE    bSwProtocol;
  BYTE    abUid[7];
  BYTE    abBatchNo[5];
  BYTE    bProductionCW;
  BYTE    bProductionYear;
} DF_VERSION_INFO;

/*
 * DF_ADDITIONAL_FILE_SETTINGS
 * ---------------------------
 * Union for returning the information supplied by the GetFileSettings command.
 * Use stDataFileSettings for Standard Data Files and Backup Data Files.
 * Use stValueFileSettings for Value Files.
 * Use stRecordFileSettings for Linear Record Files and Cyclic Record Files.
 */
typedef union
{
  struct
  {
    DWORD   eFileSize;
  } stDataFileSettings;

  struct
  {
    SDWORD  lLowerLimit;
    SDWORD  lUpperLimit;
    DWORD   eLimitedCredit;
    BYTE    bLimitedCreditEnabled;
  } stValueFileSettings;

  struct
  {
    DWORD   eRecordSize;
    DWORD   eMaxNRecords;
    DWORD   eCurrNRecords;
  } stRecordFileSettings;
} DF_ADDITIONAL_FILE_SETTINGS;

/* Data item lengths for the GetFileSettings command */
#define	DF_FILE_SETTINGS_COMMON_LENGTH	4
#define	DF_DATA_FILE_SETTINGS_LENGTH	  3
#define	DF_VALUE_FILE_SETTINGS_LENGTH	 13
#define	DF_RECORD_FILE_SETTINGS_LENGTH  9

#define	DF_APPLICATION_ID_SIZE			3
#define	DF_MAX_INFO_FRAME_SIZE			60

#endif
